from jutil.database import mysql_database_singleton, database_controller

from ScopusWp.extend import Config

from ScopusWp.scopus.config import DB_NAME

from sqlalchemy import Column, Integer, DATETIME, String, Text, BigInteger

from ScopusWp.scopus.data import ScopusAuthorProfile

import datetime
import json


class ScopusMySQLDatabase(mysql_database_singleton(
    Config.get_instance()['MYSQL']['username'],
    Config.get_instance()['MYSQL']['password'],
    Config.get_instance()['MYSQL']['host'],
    DB_NAME
)):
    pass


BASE = ScopusMySQLDatabase.get_base()


class BlackListing(BASE):

    __tablename__ = 'blacklist'

    scopusID = Column(BigInteger, primary_key=True)
    description = Column(String(200))
    created = Column(DATETIME)


class AuthorProfile(BASE):

    __tablename__ = 'profile'

    authorID = Column(BigInteger, primary_key=True)
    first_name = Column(String(500))
    last_name = Column(String(500))
    updated = Column(String(500))
    h_index = Column(String(100))
    document_count = Column(Integer)
    citation_count = Column(Integer)
    publications = Column(Text)

    def to_scopus_author_profile(self, scopus_author_profile):
        scopus_author_profile = ScopusAuthorProfile(
            self.authorID,
            self.first_name,
            self.last_name,
            self.h_index,
            self.citation_count,
            self.document_count,
            json.loads(self.publications)
        )
        return scopus_author_profile

    @staticmethod
    def from_scopus_author_profile(scopus_author_profile):
        author_profile = AuthorProfile(
            authorID=scopus_author_profile.id,
            first_name=scopus_author_profile.first_name,
            last_name=scopus_author_profile.last_name,
            updated=datetime.datetime.now(),
            h_index=scopus_author_profile.h_index,
            document_count=scopus_author_profile.document_count,
            citation_count=scopus_author_profile.citation_count,
            publications=json.dumps(scopus_author_profile.publications)
        )
        return author_profile


class DatabaseController(database_controller(ScopusMySQLDatabase)):

    def __init__(self):
        super().__init__()

    def get_blacklist(self):
        return list(map(
            lambda black_listing: int(black_listing.scopusID),
            self._select_all(BlackListing)
        ))

    def contains_profile(self, author_id):
        return self.session.query(AuthorProfile).get(author_id) is not None

    def get_profile(self, author_id):
        return self.session.query(AuthorProfile).get(author_id).first().to_scopus_author_profile()

    def add_profile(self, scopus_author_profile):
        author_profile = AuthorProfile.from_scopus_author_profile(scopus_author_profile)
        self._add_author_profile(author_profile)

    def add_blacklist(self, scopus_id, description=''):
        black_listing = BlackListing(
            scopusID=scopus_id,
            description=description,
            created=datetime.datetime.now()
        )
        self._add_blacklisting(black_listing)

    def _add_blacklisting(self, black_listing):
        self._add(
            BlackListing,
            black_listing,
            lambda x: BlackListing.scopusID == x.scopusID
        )

    def _add_author_profile(self, author_profile):
        self._add(
            AuthorProfile,
            author_profile,
            lambda x: AuthorProfile.authorID == x.authorID
        )



from ScopusWp.extend import ExtensionInstallationController

from ScopusWp.wordpress.config import PATH, NAME
from ScopusWp.wordpress.reference import BASE

import os


if __name__ == '__main__':
    install = ExtensionInstallationController(NAME)
    install.create_mysql_database('__wordpress', BASE)

    template = install.load_template(os.path.join(PATH, 'templates', 'config.ini'))
    content = template.render(
        url='',
        password='',
        username=''
    )

    install.create_file('config.ini', content)



import configparser







import logging







import os















import datetime







import pathlib







import threading















# Getting the absolute path to the folder of this very script file, so that can be used to open the config file, which







# is also located in this folder







PATH = os.path.dirname(os.path.realpath(__file__))







VERSION = '0.3.0'







PROJECT_PATH = "/home/jonas/PycharmProjects/ScopusWp/ScopusWp"
#PROJECT_PATH = '/home/jonas/IPE/ScopusWp' #















LOGGING_PATH = os.path.join(PROJECT_PATH, 'logs')







TEMPLATE_PATH = os.path.join(PATH, 'templates')















SCOPUS_LOGGING_EXTENSION = 'SCOPUS'







SQL_LOGGING_EXTENSION = 'SQL'























def _format_log_file_name(datetime_object, extension_string):















    # Formatting the string of the date







    date_string = _format_date(datetime_object)







    # Joining the date string with the string extension







    file_name_string = '-'.join([date_string, 'log'])







    file_name_string += '.txt'















    return file_name_string























def _format_date(datetime_object):







    date_string = '{}-{}-{}'.format(datetime_object.year, datetime_object.month, datetime_object.day)







    return date_string























def get_log_file_path(datetime_object, extension_string):







    file_name = _format_log_file_name(datetime_object, extension_string)







    file_path = os.path.join(LOGGING_PATH, file_name)







    return file_path























def log_exists(datetime_object, extension_string):







    file_path = get_log_file_path(datetime_object, extension_string)















    return os.path.exists(file_path) and os.path.isfile(file_path)























def init_logging():







    datetime_object = datetime.datetime.now()







    file_path = get_log_file_path(datetime_object, '')







    # Checking if the log already exists, if not







    if log_exists(datetime_object, ''):







        file_mode = 'a'







        with open(file_path, 'a') as file:







            file.write('\n\nSTARTING A NEW SESSION:\n\n')







    else:







        file_mode = 'w+'















    logging.basicConfig(level=logging.INFO,







                        filename=file_path,







                        filemode=file_mode,







                        format='%(asctime)s %(name)-40s %(levelname)-8s %(message)s')























class LoggingController:















    DATE_FORMAT = '%Y-%m-%d'







    DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S'















    def __init__(self):







        self.config = Config.get_instance()















        self.log_file_path = pathlib.Path(LOGGING_PATH) / self.create_log_file_name()















    def init(self):







        self.prepare_log_file(self.log_file_path)















    def close(self):







        self.post_process_log_file(self.log_file_path)















    def prepare_log_file(self, path):















        # Checking if the path exists and if it does, that means there was a log written on that day already, then







        # opening the file in append mode







        # Getting the current datetime string







        datetime_object = datetime.datetime.now()







        datetime_string = datetime_object.strftime(self.DATETIME_FORMAT)















        line_string = '\n\nSTARTING A NEW SESSION @ "{}"\n\n'.format(datetime_string)















        file_mode = 'w+'







        with path.open(mode=file_mode) as file:







            file.write(line_string)















        file_path = str(path)







        logging.basicConfig(







            level=logging.DEBUG,







            filename=file_path,







            filemode='a',







            format='%(asctime)s %(name)-40s %(levelname)-8s %(message)s'







        )















    def post_process_log_file(self, path):







        datetime_object = datetime.datetime.now()







        datetime_string = datetime_object.strftime(self.DATETIME_FORMAT)















        line_string = '\n\nCLOSING SESSION @ "{}"\n\n'.format(datetime_string)















        with path.open(mode='a') as file:







            file.write(line_string)















    def create_activity_file_name(self):







        # Creating the date string from the current datetime object







        datetime_object = datetime.datetime.now()







        date_string = datetime_object.strftime(self.DATE_FORMAT)







        # The base name extension for the file name is saved as a entry in the config file







        base_name_string = self.config['LOGGING']['activity_log_name']















        # Joining the date string and the base string into the log file name







        name_string = '{}-{}.txt'.format(







            date_string,







            base_name_string







        )















        return name_string















    def create_log_file_name(self):







        # Creating the date string from the current datetime object







        datetime_object = datetime.datetime.now()







        date_string = datetime_object.strftime(self.DATE_FORMAT)







        # The base name extension for the file name is saved as a entry in the config file







        base_name_string = self.config['LOGGING']['debug_log_name']















        # Joining the date string and the base string into the log file name







        name_string = '{}-{}.txt'.format(







            date_string,







            base_name_string







        )















        return name_string























class LoggingObserver(threading.Thread):















    def __init__(self, logger_name):















        self.logger = logging.getLogger(logger_name)























class Config:







    """







    A singleton class for the access to the config object







    """







    _instance = None















    def __init__(self):







        pass















    @staticmethod







    def get_instance():















        if Config._instance is None:







            Config._instance = Config._create_instance()















        return Config._instance















    @staticmethod







    def _create_instance():







        # Getting the path to the config file







        path = Config._config_path()







        # Creating the config parser object and returning it







        config = configparser.ConfigParser()







        config.read(path)















        return config















    @staticmethod







    def _config_path():







        # Joining the config path







        path = os.path.join(PROJECT_PATH, 'config.ini')







        return path



































import logging
import pathlib
import configparser
import sys
import getopt
import jinja2
import os

from ScopusWp.database import MySQLDatabase

from ScopusWp.config import PATH, Config

from jutil.installation import PathLookupPrompt
from jutil.installation import create_database_mysql, create_sqlalchemy_mysql

from sqlalchemy.orm import sessionmaker, Session


class ExtensionInstallationController:
    """
    DEV
    -> One could add support for creating additional sqlite databases, but if that is the case the main class would be
    too overloaded and one would have to make a separate class to handle the databases
    """
    OPTIONS_FORMAT_SHORT = 'p:'
    OPTIONS_FORMAT_LONG = ['project_path=']

    def __init__(self, extension_name):
        self.name = extension_name
        self.options, self.arguments = getopt.getopt(sys.argv[1:], self.OPTIONS_FORMAT_SHORT, self.OPTIONS_FORMAT_LONG)
        self.project_path = self.get_project_path()

        self.path = pathlib.Path(self.project_path) / extension_name
        # Creating the folder, if it does not already exists
        self._create_extension_folder()

        self.explorer = PubControlExplorer(self.project_path)

        self._engine = None
        self._session_maker = None

        self._session = None

    @property
    def session(self):
        if self._session_maker is None:
            raise AttributeError('No database has been created yet')
        if self._session is None:
            self._session = self._session_maker()
        return self._session

    def create_file(self, name, content):
        """
        Creates a new file in the local folder of the extension inside of the project root and writes the given content
        into it.
        :param name: The name of the file
        :param content: The content to be written into the file
        :return: void
        """
        file_path = self.path / name
        if not file_path.exists():
            with file_path.open(mode='w+') as file:
                file.write(content)
                file.flush()

    def load_template(self, path):
        """
        If given the path to a jinja2 template file, this method will load the files contents into a jinja template
        and returns this template object.
        :param path: The string path to the template file
        :return: jinja2.Template
        """
        template_path = pathlib.Path(path)
        with template_path.open() as file:
            return jinja2.Template(file.read())

    def create_mysql_database(self, name, declarative):
        """
        Given the name of the database and the sqlalchemy declarative base this method will create all the tables
        according to sqlalchemy models. If there is no database with the given name yet a new one will be created
        IMPORTANT
        Make sure, that the module, which contains all the actual model classes for sqlalchemy has been imported or is
        in the scope, when this method is being called, otherwise it will not work
        :param name: The name of the database, that is supposed to be used for the sqlalchemy tables
        :param declarative: The declarative base object of sqlalchemy, that was used to create all the model classes
        :return:
        """
        # Attempting to create the database
        try:
            create_database_mysql(
                self.explorer.mysql_username,
                self.explorer.mysql_password,
                self.explorer.mysql_host,
                name,
            )
        except Exception as e:
            print("{} {}".format(type(e), str(e)))
        # Creating the tables for the sqlalchemy models
        try:
            self._engine = create_sqlalchemy_mysql(
                self.explorer.mysql_username,
                self.explorer.mysql_password,
                self.explorer.mysql_host,
                name,
                declarative
            )
            self._session_maker = sessionmaker(bind=self._engine)
        except Exception as e:
            print("{} {}".format(type(e), str(e)))

    def get_project_path(self):
        """
        Attempts to get the project path from the command line options called with the script. In case the project
        path of PubControl was not given through the command line, prompts the user to input the path manually, as the
        path is absolutely necessary for the installation to work.
        :return: string path
        """
        if len(sys.argv[1:]) != 0:
            # Checking the options passed to the script, if it contains the path option returning that for te path
            for opt, arg in self.options:
                if opt in ['-p', '--project_path']:
                    return arg

        # In case there was no path in the command line call, prompts the user to manually input the path
        prompt = PathLookupPrompt()
        return prompt.get()

    def _create_extension_folder(self):
        """
        Every extension for PubControl gets its own folder in the PubControl project root and there it can install all
        configuration/output files w/e the user has to interact with in some way eventually.
        This function creates a new such folder, if it does not already exists.
        :raises NotADirectoryError: When the path to the extension folder already exists, but is not a folder
        :return: void
        """
        # Checking if the folder already exists, if it already exists, does not create the folder
        if self.path.exists():
            # If the path exists, but is not a folder, but a file or something else, that is a problem and a error
            if not self.path.is_dir():
                raise NotADirectoryError((
                    'The extension "{}" to PUBCONTROL could not be installed. '
                    'path to extension root already exists, but is not a folder'
                ).format(self.name))
        else:
            # Creating the folder
            self.path.mkdir()


class PubControlExplorer:

    def __init__(self, project_path):
        self.path_string = project_path
        self.path = pathlib.Path(project_path)

        # Will contain the configparser.ConfigParser object after the project folder has been read
        self.config = None

        # This is the session object for the sqlalchemy mysql database of the pub control project, this database
        # contains all the publications
        self.session = MySQLDatabase.get_session()

        self.check_path()
        # Actually reading the config file into the variable
        self.read_config_file()

    def read_config_file(self):
        """
        Creates a new config parser object and reads the contents of the config.ini in the project folder to it.
        :return: void
        """
        config_file_path = str(self.path / "config.ini")

        self.config = configparser.ConfigParser()
        self.config.read(config_file_path)

    @property
    def mysql_username(self):
        """
        The string username for the mysql database
        :return: string
        """
        return self.config['MYSQL']['username']

    @property
    def mysql_password(self):
        """
        The string password for mysql database and the given user
        :return: string
        """
        return self.config['MYSQL']['password']

    @property
    def mysql_host(self):
        """
        The string, specifying the host argument for the mysql database
        :return: string
        """
        return self.config['MYSQL']['host']

    def check_path(self):
        """
        Checks whether the path given exists and if it is a folder. In case it is not raises an error

        :raises EnvironmentError: In case the given project path is not correct
        :return: void
        """
        if not(self.path.exists() and self.path.is_dir()):
            raise EnvironmentError('The given PubControl Project path is not correct')


class DataSource:

    def __init__(self, controller):
        self.controller = controller
        self.logger = logging.getLogger('source-{}'.format(self.name))

    def fetch(self):
        raise NotImplementedError()

    @property
    def name(self):
        raise NotImplementedError()


class DataSink:

    def __init__(self, controller):
        self.controller = controller
        self.logger = logging.getLogger('sink-{}'.format(self.name))

    @property
    def run(self):
        raise NotImplementedError()

    @property
    def register(self):
        return self.controller.register

    @property
    def config(self):
        return self.controller.config

    @property
    def name(self):
        raise NotImplementedError()
